//
// Display.cc
//
// Implementation of Display
// Takes results of search and fills in the HTML templates
//
//
#if RELEASE
static char RCSid[] = "$Id: PyDisplay.cc,v 1.4 2001/12/08 23:20:19 erickson Exp erickson $";
#endif

#include "htsearch.h"
#include "PyDisplay.h"
#include "ResultMatch.h"
#include "WeightWord.h"
#include "StringMatch.h"
#include "QuotedStringList.h"
#include "URL.h"
#include <fstream.h>
#include <stdio.h>
#include <ctype.h>
#include <syslog.h>
#include <locale.h>
#include "HtURLCodec.h"
#include "HtWordType.h"

extern void encodeSGML(String &str);

//*****************************************************************************
//
PyDisplay::PyDisplay(char *indexFile, char *docFile) 
	: Display(indexFile, docFile)
{
}

//*****************************************************************************
PyDisplay::~PyDisplay()
{
//	docIndex->Close();
//	delete docIndex;
}

//*****************************************************************************
//
void
PyDisplay::display(PyObject* pVarsList, PyObject* pHitList, int pageNumber)
{
    List		*matches = buildMatchList();
    int			currentMatch = 0;
    int			numberDisplayed = 0;
    ResultMatch		*match = 0;
    int			number = config.Value("matches_per_page");
    if (number <= 0)
	number = 10;
    int			startAt = (pageNumber - 1) * number;

    if (config.Boolean("logging"))
    {
        logSearch(pageNumber, matches);
    }

    setVariables(pVarsList, pageNumber, matches);
	
    //
    // The first match is guaranteed to have the highest score of
    // all the matches.  We use this to compute the number of stars
    // to display for all the other matches.
    //
    match = (ResultMatch *) (*matches)[0];
    if (!match)
    {
	//
	// No matches.
	//
        delete matches;
	// PY - return empty buffer.
	return;
    }
    	
    matches->Start_Get();
    while ((match = (ResultMatch *)matches->Get_Next()) &&
	   numberDisplayed < number)
    {
	if (currentMatch >= startAt)
	{
	    match->setRef(docDB[match->getURL()]);
	    DocumentRef	*ref = match->getRef();
	    if (!ref)
		continue;	// The document isn't present for some reason
	    ref->DocAnchor(match->getAnchor());
	    ref->DocScore(match->getScore());
	    addMatchToList(pHitList, match,currentMatch+1);
	    numberDisplayed++;
	    match->setRef(NULL);
	    delete ref;
	}
	currentMatch++;
    }

    delete matches;
}

int PyDisplay::putStringInDict(PyObject* pDict, 
		               const char* sKey, const char* sValue)
{
    PyObject* pKey;
    PyObject* pValue;
    int i;
    
    /* Create a dictionary entry */
    pKey=PyString_FromString (sKey);
    pValue=PyString_FromString (sValue);
    i = PyDict_SetItem (pDict, pKey, pValue);
    Py_DECREF(pKey);
    Py_DECREF(pValue);
    
    return i;
}

int PyDisplay::putStringInDict(PyObject* pDict, 
		               const char* sKey, String* sValue)
{
    PyObject* pKey;
    PyObject* pValue;
    int i;
    
    /* Create a dictionary entry */
    pKey=PyString_FromString (sKey);
    pValue=PyString_FromString (sValue->get());
    i = PyDict_SetItem (pDict, pKey, pValue);
    Py_DECREF(pKey);
    Py_DECREF(pValue);
    
    return i;
}

int PyDisplay::putLongInDict(PyObject* pDict, const char* sKey, const long lValue)
{
    PyObject* pKey;
    PyObject* pValue;
    int i;
    
    /* Create a dictionary entry */
    pKey=PyString_FromString (sKey);
    pValue=PyLong_FromLong (lValue);
    i = PyDict_SetItem (pDict, pKey, pValue);
    Py_DECREF(pKey);
    Py_DECREF(pValue);
    
    return i;
}

int PyDisplay::putFloatInDict(PyObject* pDict, const char* sKey, const double fValue)
{
    PyObject* pKey;
    PyObject* pValue;
    int i;
    
    /* Create a dictionary entry */
    pKey=PyString_FromString (sKey);
    pValue=PyFloat_FromDouble (fValue);
    i = PyDict_SetItem (pDict, pKey, pValue);
    Py_DECREF(pKey);
    Py_DECREF(pValue);
    
    return i;
}


//*****************************************************************************
int
PyDisplay::addMatchToList(PyObject* pList, ResultMatch *match, int current)
{
    PyObject* pDict=PyDict_New();
    String	*str = 0;

    if (!pDict)
        return -1;
	
    DocumentRef	*ref = match->getRef();

    // get the URL
    char    *url = match->getURL();
    putStringInDict(pDict, "URL", url);


    int     iA = ref->DocAnchor();
    
    String  *anchor = 0;
    int             fanchor = 0;
    if (iA > 0)             // if an anchor was found
      {
	List    *anchors = ref->DocAnchors();
	if (anchors->Count() >= iA)
	  {
	    anchor = new String();
	    fanchor = 1;
	    *anchor << "#" << ((String*) (*anchors)[iA-1])->get();
            putStringInDict(pDict, "ANCHOR", anchor);
	  }
      }
    
    //
    // no condition for determining excerpt any more:
    // we need it anyway to see if an anchor is relevant
    //
    int first = -1;
    String urlanchor(url);
    if (anchor)
      urlanchor << anchor;
    putStringInDict(pDict, "EXCERPT", excerpt(ref, urlanchor, fanchor, first));


    //
    // anchor only relevant if an excerpt was found, i.e.,
    // the search expression matches the body of the document
    // instead of only META keywords.
    //
    /*
    if (first < 0)
      {
	vars.Remove("ANCHOR");
      }
    */
    
    putLongInDict(pDict, "SCORE", match->getScore());
    putLongInDict(pDict, "CURRENT", current);
    char	*title = ref->DocTitle();
    if (!title || !*title)
      {
	if ( strcmp(config["no_title_text"], "filename") == 0 )
	  {
	    // use actual file name
	    title = strrchr(url, '/');
	    if (title)
	      {
		title++; // Skip slash
		str = new String(form("[%s]", title));
	      }
	    else
	      // URL without '/' ??
	      str = new String("[No title]");
	  }
	else
	  // use configure 'no title' text
	  str = new String(config["no_title_text"]);
      }
    else
      str = new String(title);

    putStringInDict(pDict, "TITLE", str);

    {
        // How many stars does this match have?
        double	score;
        int	nStars;

        if (maxScore != 0)
        {
    	score = ref->DocScore() / (double)maxScore;
        }
        else
        {
    	maxScore = ref->DocScore();
    	score = 1;
        }
        nStars = int(score * (maxStars - 1) + 0.5) + 1;
        putLongInDict(pDict, "NUMSTARS", nStars);
    }

    putLongInDict(pDict, "SIZE", ref->DocSize());
    putLongInDict(pDict, "SIZEK", (ref->DocSize() + 1023) / 1024);

    if (maxScore != 0)
      {
	int percent = (int)(ref->DocScore() * 100 / (double)maxScore);
	if (percent <= 0)
	  percent = 1;
        putFloatInDict(pDict, "PERCENT", percent);
      }
    else
        putFloatInDict(pDict, "PERCENT", 100);
    
    {
	str = new String();
	char		buffer[100];
	time_t		t = ref->DocTime();
	if (t)
	{
	    struct tm	*tm = localtime(&t);
	    char	*datefmt = config["date_format"];
	    char	*locale  = config["locale"];
	    if (!datefmt || !*datefmt)
	      {
		if (config.Boolean("iso_8601"))
		    datefmt = "%Y-%m-%d %H:%M:%S %Z";
		else
		    datefmt = "%x";
	      }
	    if ( locale && *locale )
	      {
		setlocale(LC_TIME,locale);
	      }
	    strftime(buffer, sizeof(buffer), datefmt, tm);
	    *str << buffer;
	}
        putStringInDict(pDict, "MODIFIED", str);
    }
	
    putLongInDict(pDict, "HOPCOUNT", ref->DocHopCount());
    putLongInDict(pDict, "DOCID", ref->DocID());
    putLongInDict(pDict, "BACKLINKS", ref->DocBackLinks());
	
    {
	str = new String();
	List	*list = ref->Descriptions();
	int		n = list->Count();
	for (int i = 0; i < n; i++)
	{
	    *str << ((String*) (*list)[i])->get() << "<br>\n";
	}
        putStringInDict(pDict, "DESCRIPTIONS", str);
        String *description = new String();
        if (list->Count())
            *description << ((String*) (*list)[0]);
        putStringInDict(pDict, "DESCRIPTION", description);
    }

    /* Insert the dictionary entry into the list */
    PyList_Append (pList, pDict);
    Py_DECREF(pDict);


    return 0;
}

//*****************************************************************************
//  this seems to set variables that are consistent across the Whole returned
//  page. I'll put them in a separate dictionary.
void
PyDisplay::setVariables(PyObject* pDict, int pageNumber, List *matches)
{
    String	tmp;
    int		i;
    int		nMatches = 0;

    if (matches)
	nMatches = matches->Count();
	
    int		matchesPerPage = config.Value("matches_per_page");
    if (matchesPerPage <= 0)
	matchesPerPage = 10;
    int		nPages = (nMatches + matchesPerPage - 1) / matchesPerPage;

    if (nPages > config.Value("maximum_pages", 10))
	nPages = config.Value("maximum_pages", 10);
    if (nPages < 1)
	nPages = 1;			// We always have at least one page...
    putStringInDict(pDict, "MATCHES_PER_PAGE", config["matches_per_page"]);
    putStringInDict(pDict, "MAX_STARS", config["max_stars"]);
//    putStringInDict(pDict, "CONFIG", config["config"]);
    putStringInDict(pDict, "VERSION", config["version"]);
    putStringInDict(pDict, "RESTRICT", config["restrict"]);
    putStringInDict(pDict, "EXCLUDE", config["exclude"]);
    putStringInDict(pDict, "KEYWORDS", config["keywords"]);
    if (mystrcasecmp(config["match_method"], "and") == 0)
        putStringInDict(pDict, "MATCH_MESSAGE", "all");
    else if (mystrcasecmp(config["match_method"], "or") == 0)
        putStringInDict(pDict, "MATCH_MESSAGE", "some");
    putLongInDict(pDict, "MATCHES", nMatches);
    putStringInDict(pDict, "PLURAL_MATCHES", nMatches == 1 ? (char *)"" : (char *)"s");
    putLongInDict(pDict, "PAGE", pageNumber);
    putLongInDict(pDict, "PAGES", nPages);
    putLongInDict(pDict, "FIRSTDISPLAYED",
		 (pageNumber - 1) * matchesPerPage + 1);
//    if (nPages > 1)
//        putStringInDict(pDict, "PAGEHEADER", config["page_list_header"]);
//    else
//        putStringInDict(pDict, "PAGEHEADER", config["no_page_list_header"]);
	
    i = pageNumber * matchesPerPage;
    if (i > nMatches)
	i = nMatches;
    putLongInDict(pDict, "LASTDISPLAYED", i);

    String	*str;

    StringList form_vars(config["allow_in_form"], " \t\r\n");
    String* key;
    for (i= 0; i < form_vars.Count(); i++)
    {
      if (config[form_vars[i]])
      {
	key= new String(form_vars[i]);
	key->uppercase();
        putStringInDict(pDict, key->get(), config[form_vars[i]]);
      }
    }
    return;
}



